class Endpoints:
    # Base Urls
    BASE_URL_API = "https://axiom.trade/api"
    BASE_URL = "https://axiom.trade"

    # Endpoints
    ENDPOINT_GET_BALANCE = "/sol-balance"
    ENDPOINT_GET_BATCHED_BALANCE = "/batched-sol-balance"
    ENDPOINT_BUY_TOKEN = "/buy"
    ENDPOINT_SELL_TOKEN = "/sell"
    ENDPOINT_SEND_TRANSACTION = "/send-transaction"
    ENDPOINT_GET_TOKEN_BALANCE = "/token-balance"