class AAllBaseUrls:
    BASE_URL = "https://api.axiomtrade.com"
    BASE_URL_v2 = "https://api2.axiom.trade"
    BASE_URL_v3 = "https://api3.axiom.trade"
    BASE_URL_v4 = "https://api4.axiom.trade"
    BASE_URL_v5 = "https://api5.axiom.trade"
    BASE_URL_v6 = "https://api6.axiom.trade"
    BASE_URL_v7 = "https://api7.axiom.trade"
    BASE_URL_v8 = "https://api8.axiom.trade"
    BASE_URL_v9 = "https://api9.axiom.trade"
    BASE_URL_v10 = "https://api10.axiom.trade"

class AxiomTradeApiUrls:
    LOGIN_STEP1 = f"/login-password-v2"
    LOGIN_STEP2 = f"/login-otp"
    LOGOUT = f"/auth/logout"
    USER_INFO = f"/user/info"
    SUBSCRIBE_NEW_TOKENS = f"/ws/subscribe/new-tokens"
    SUBSCRIBE_ORDERS = f"/ws/subscribe/orders"
    SUBSCRIBE_POSITIONS = f"/ws/subscribe/positions"